#!/usr/bin/env python3
# coding:utf-8
import os
import sys
import pandas as pd
import json
from collections import defaultdict

# 获得hpo和omim表型编号的对应关系：omim.phe_hpo.phe.json，hpo和omim表型编号的对应关系，一个hpo表型编号可能对应多个omim表型编号
# 获得omim表型编号和基因的对应关系：omim_gene.json,一个omim表型编号一般对应多个基因
# 获得基因和表型的对应关系：inh_gene_phenotype.json，基因对应的遗传方式和omim表型

dirpath = os.path.dirname(sys.argv[0])
package_dirpath = os.path.dirname(dirpath)
omim_phe_hpo_phe = os.path.join(package_dirpath, 'data/omim.phe_hpo.phe')
omim_phe_hpo_phe_json = os.path.join(package_dirpath, 'data/omim.phe_hpo.phe.json')
omimdata = os.path.join(package_dirpath, 'data/omimdata.xlsx')
omim2 = os.path.join(package_dirpath, 'data/omim2.csv')
omim3 = os.path.join(package_dirpath, 'data/omim3.txt')
genemap2 = os.path.join(package_dirpath, 'data/genemap2.txt')
inh_gene_phenotype_csv = os.path.join(package_dirpath, 'data/inh_gene_phenotype.csv')
inh_gene_phenotype_json = os.path.join(package_dirpath, 'data/inh_gene_phenotype.json')
omim_gene_json = os.path.join(package_dirpath, 'data/omim_gene.json')


def hpo():
    dic = defaultdict(list)
    with open(omim_phe_hpo_phe, 'r') as f:
        for i in f:
            line = i.strip().split()
            omim = line[0]
            hp = line[1].split(':')[1]
            dic[hp].append(omim)
    with open(omim_phe_hpo_phe_json, 'w') as f:  # hpo编号和omim编号对应关系,一对多
        json.dump(dic, f)


def omim():  # 获得基因和相应的omim表型号
    df = pd.read_excel(omimdata, sheet_name='Sheet1', header=0)
    col = list(df.columns)
    phe = [v for i, v in enumerate(col) if v.find('Phenotypes') != -1]
    phenotye = phe[:]
    phe.insert(0, 'Gene Symbols')
    df = df[phe]
    df = df.fillna('.')
    for i in range(len(df)):
        for j in phenotye:
            data = df.loc[i, j]
            if data == '.':
                continue
            else:
                data = data.split(',')
                if len(data) == 1:
                    df.loc[i, j] = '.'
                else:
                    data = data[-1].split(' ')[1]
                    if data.isdigit():
                        df.loc[i, j] = data
                    else:
                        df.loc[i, j] = '.'
    df.to_csv(omim2, index=False, header=True, sep='\t')


def ff():
    with open(omim2, 'r') as f:
        next(f)
        with open(omim3, 'w') as p:  # 基因与omim表型编号对应关系
            for i in f:
                line = i.strip().split('\t')
                gene = line[0].split(',')[0]
                omim = line[1:]
                for j in omim:
                    if j != '.':
                        p.write(gene+'\t'+j+'\n')


def final():
    dic = defaultdict(list)
    df = pd.read_table(omim3, header=None)
    df.columns = ['gene', 'omim_phe']
    df = df.applymap(str)
    for i in range(len(df)):
        omim = df.loc[i, 'omim_phe']
        gene = df.loc[i, 'gene']
        if gene not in dic[omim]:
            dic[omim].append(gene)  # omim编号和基因对应关系，一对多
    return dic


def ad_gene_phe():  # 得到gene:[inheritance,phenotype]对应关系，进行后面的omim数据注释
    df1 = pd.read_excel(omimdata, header=0, sheet_name='Sheet1')
    df1 = df1[['Gene Symbols', 'MOI']]
    df1['Approved Symbol'] = ''
    df1 = df1.fillna('.')
    df1['Approved Symbol'] = df1['Gene Symbols'].map(lambda x: x.split(',')[0])
    df1 = df1.drop('Gene Symbols', axis=1)
    df2 = pd.read_table(genemap2, header=3)
    df2 = df2[['Approved Symbol', 'Phenotypes']]
    df = pd.merge(df1, df2)
    df.columns = ['inheritance', 'gene', 'phenotype']
    df = df.fillna('.')
    df.to_csv(inh_gene_phenotype_csv, header=True,
              index=False, sep='\t', encoding='utf_8')


# 获得基因和表型的对应关系：inh_gene_phenotype.json
def gene_phe():  # 存成json格式
    dic = {}
    with open(inh_gene_phenotype_csv, 'r') as f:
        next(f)
        for i in f:
            line = i.strip().split('\t')
            gene = line[1]
            inheritance = line[0]
            phenotype = line[2]
            dic[gene] = [inheritance, phenotype]
    with open(inh_gene_phenotype_json, 'w') as f:
        json.dump(dic, f)


def main():
    hpo()
    omim()
    ff()
    dic = final()
    # 根据hpo号得到omim表型号，再根据omim表型号得到gene
    # omim.phe_hpo.phe.json->omim_gene.json
    with open(omim_gene_json, 'w') as f:
        json.dump(dic, f)
    ad_gene_phe()
    gene_phe()



#!/usr/bin/env python3
#coding:utf-8
import sys
import os
import pandas as pd

pos_list=[0,2,3,4,5,6,7,8,9,12,20,21,22,23,24,26,27,28,29,34,36,37,38,39,40]
def report(path):
    r_list=[]
    path=os.path.join(path,'coverage.report')
    with open(path,'r') as f:
        for i in f:
            if i.startswith('#'):
                continue
            else:
                line=i.strip()
                r_list.append(line)
    f_list=[r_list[i] for i in pos_list]
    return f_list


def write_cov(f_list):
    output=os.path.basename(path)
    with open(output+'_simple_covreport.txt','w') as f:
        for line in f_list:
            f.write(line + '\n')
    output=os.path.join(path,'depth.tsv.gz')
    df=pd.read_table(output,header=0)
    out=df['Raw Depth'].describe()
    out.to_csv(path+'_describe.txt')


if __name__ == '__main__':
    path=sys.argv[1]
    path=os.path.abspath(path)
    f_list=report(path)
    write_cov(f_list)






#!/usr/bin/env python3
# coding:utf-8


import os
import sys
import pysam


class Trio(object):
    # child:father:mother:genetic_pattern
    t0 = {0: {0: {0: 'anyway', 1: 'anyway', 2: 'back_mutation'}, 1: {0: 'anyway', 1: 'anyway',
                                                                     2: 'back_mutation'}, 2: {0: 'back_mutation', 1: 'back_mutation', 2: 'back_mutation'}}}
    t1 = {1: {0: {0: 'denovo', 1: 'M_HET', 2: 'M_HOM'}, 1: {0: 'F_HET',
                                                            1: 'Unknown', 2: 'M_HOM'}, 2: {0: 'F_HOM', 1: 'F_HOM', 2: 'back_mutation'}}}
    t2 = {2: {0: {0: 'denovo', 1: 'denovo', 2: 'denovo'}, 1: {0: 'denovo', 1: 'F_HET_M_HET',
                                                              2: 'F_HET_M_HOM'}, 2: {0: 'denovo', 1: 'F_HOM_M_HET', 2: 'F_HOM_M_HOM'}}}
    trio = {}
    trio.update(t0)
    trio.update(t1)
    trio.update(t2)

    def __init__(self, vcf, child, father, mother):
        self.vcf = vcf
        self.child = child
        self.father = father
        self.mother = mother

    def get_child_father_mother_GT(self, rec):
        for sampleRecorde in rec.samples.values():
            if sampleRecorde.name == self.father:
                father_gt = '/'.join(str(s) for s in sampleRecorde['GT'])
            elif sampleRecorde.name == self.mother:
                mother_gt = '/'.join(str(s) for s in sampleRecorde['GT'])
            else:
                child_gt = '/'.join(str(s) for s in sampleRecorde['GT'])

        return child_gt, father_gt, mother_gt

    def parse_vcf(self):
        _tran = {'0/0': 0, '0/1': 1, '1/1': 2, '0|0': 0, '0|1': 1, '1|1': 2}
        vcf = pysam.VariantFile(self.vcf)
        vcf.header.info.add('genetic_pattern', '.', 'String',
                            "genetic pattern made by lianlin")
        sample_list = list(vcf.header.samples)
        assert len(
            sample_list) == 3, 'only support trio sample,the number of sample should equal to 3'
        assert self.child in sample_list and self.father in sample_list and self.mother in sample_list, 'child,father,mother mismatch'
        for rec in vcf.fetch():
            child_gt, father_gt, mother_gt = self.get_child_father_mother_GT(
                rec)
            child_gt = _tran.get(child_gt, 'invalid')
            father_gt = _tran.get(father_gt, 'invalid')
            mother_gt = _tran.get(mother_gt, 'invalid')
            if child_gt == 'invalid' or father_gt == 'invalid' or mother_gt == 'invalid':
                # 大写Unknown代表三人都是杂合情况，小写unknown代表基因型无法判断的情况
                genetic_pattern = 'unknown'
                rec.info['genetic_pattern'] = genetic_pattern
            else:
                genetic_pattern = self.trio.get(child_gt, {}).get(
                    father_gt, {}).get(mother_gt, 'unknown')
                rec.info['genetic_pattern'] = genetic_pattern
            print(genetic_pattern)
            yield rec
        vcf.close()

    def vcf_out(self):
        module = sys.argv[1]
        outprefix = module + '.' + os.path.basename(self.vcf)
        vcf_in = pysam.VariantFile(self.vcf)
        vcf_in.header.add_line(
            '##INFO=<ID=genetic_pattern,Number=.,Type=String,Description="genetic pattern made by lianlin">')
        vcf_in.header.add_line('##Conmand={}>'.format(' '.join(sys.argv)))
        vcf_out = pysam.VariantFile(outprefix, 'w', header=vcf_in.header)
        for rec in self.parse_vcf():
            vcf_out.write(rec)
        print('the result is {}'.format(outprefix))
        vcf_in.close()
        vcf_out.close()


def main(p_dict):
    vcf = p_dict['vcf']
    child = p_dict['child']
    father = p_dict['father']
    mother = p_dict['mother']
    result = Trio(vcf, child, father, mother)
    result.vcf_out()



#!/usr/bin/env python3
# coding:utf-8

import argparse
import sys
import trio
import subprocess
import omim_match
import database_format
import vcf_split
import wes
import pedigree
import infer_gender

parser = argparse.ArgumentParser(
    description='whole exome analysis workflow made by lianlin')
subparsers = parser.add_subparsers(
    help='Select pywes action among the following options: ', dest='pygenetic_action')
parser_wes = subparsers.add_parser('wes', help='wes analysis pipeline.')
parser_carrier = subparsers.add_parser(
    'carrier', help='vcf file further annotation and filter.')
parser_trio = subparsers.add_parser(
    'trio', help='predict inheritance pattern when use trio vcf,should only contain three samples belong to the same family.')
parser_pedigree = subparsers.add_parser(
    'pedigree', help='pedigree vcf file further annotation and filter.')
parser_panel = subparsers.add_parser(
    'panel', help='annotation result filter according to gene panel.')
parser_infer_gender = subparsers.add_parser(
    'gender', help='sex prediction.')
parser_database = subparsers.add_parser(
    'database', help='database update.')

# wes argument
parser_wes.add_argument(
    '-i', '--input', help='configure file', required=True)
parser_wes.add_argument(
    '-r', '--run', choices={'on','off'},help='run on or run off,default off',default='off')

# carrier arguments

parser_carrier.add_argument(
    '-vcf', '--vcf', help='annoted vcf file from annovar software', required=True)
parser_carrier.add_argument(
    '-r', '--rank', help='variant rank according to ACMG guideline', required=False)
# parser_carrier.add_argument(
#     '-t', '--variant_type', help='SNP or INDEL', required=True)

# trio arguments
parser_trio.add_argument('-vcf', '--vcf', help='vcf file', required=True)
parser_trio.add_argument(
    '-c', '--child', help='child sample ID', required=True)
parser_trio.add_argument(
    '-f', '--father', help='father sample ID', required=True)
parser_trio.add_argument(
    '-m', '--mother', help='mother sample ID', required=True)
    
# pedigree arguments
parser_pedigree.add_argument(
    '-vcf', '--vcf', help='annoted vcf file from annovar software', required=True)
parser_pedigree.add_argument(
    '-rank', '--rank', help='variant rank according to ACMG guideline', required=False)
parser_pedigree.add_argument(
    '-c', '--child', help='child sampleID', required=True)
parser_pedigree.add_argument(
    '-f', '--father', help='father sampleID', required=True)
parser_pedigree.add_argument(
    '-m', '--mother', help='mother sampleID', required=True)

# panel arguments
parser_panel.add_argument(
    '-hpo', '--hpo', help='hpo phenotype code', required=True)
parser_panel.add_argument(
    '-c', '--csv', help='csv list,each csv each row', required=True)


# infeer_gender arguments
parser_infer_gender.add_argument(
    '-b', '--bam', help='bam files ,you can also use wildcard character,e.g dedup.bam', default='.bam')
parser_infer_gender.add_argument(
    '-w', '--working_space', help='bam files will be collected in provided working directory', required=True)
# database arguments
# actually,no argument


def main():
    if len(sys.argv) == 1:
        parser.print_help(sys.stderr)
        sys.exit(1)
    parameters = parser.parse_args()
    p_dict = vars(parameters)
    action = p_dict['pygenetic_action']
    if action == 'wes':
        wes.main(p_dict)
    elif action == 'carrier':
        vcf_split.main(p_dict)
    elif action == 'trio':
        trio.main(p_dict)
    elif action == 'pedigree':
        pedigree.main(p_dict)
    elif action == 'panel':
        omim_match.main(p_dict)
    elif action == 'gender':
        infer_gender.main(p_dict)
    elif action == 'database':
        database_format.main()


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# coding:utf-8

import os
import time
import sys
import subprocess
from collections import defaultdict


class Wes(object):

    trimmomatic = "/share/data1/local/bin/trimmomatic-0.38.jar"
    gatk = "/share/data1/src/gatk/gatk"
    seqtk = "/share/data1/src/bwa/bwakit/seqtk"
    alt = "/share/data1/genome/hs38DH.fa.alt"
    annovar = "/share/data1/src/annovar"
    Intervar = "/share/data3/lianlin/soft/InterVar-master/Intervar2.py"
    bamdst = "/share/data3/lianlin/soft/bamdst/bamdst"
    dbsnp = "/share/data1/PublicProject/GATK_bundle/dbsnp150_chr.vcf.gz"
    indel = "/share/data1/PublicProject/GATK_bundle/Mills_and_1000G_gold_standard.indels.hg38.vcf.gz"
    snp = "/share/data1/PublicProject/GATK_bundle/1000G_phase1.snps.high_confidence.hg38.vcf.gz"
    bed = "/share/data2/leon/exom_bed/S07084713_Regions.bed"
    interval = "/share/data2/leon/exom_bed/list.interval_list"
    reference = "/share/data1/genome/hs38DH.fa"
    ip = 100

    def __init__(self, input):
        self.input = input
        self.group()

    def create_working_space(self):
        try:
            if not os.path.exists(self.working_space):
                os.mkdir(self.working_space)
                sys.stdout.write('mkdir {}  directory\n'.format(
                    self.working_space))
            else:
                os.removedirs(self.working_space)
                os.mkdir(self.working_space)
                sys.stdout.write('delete this empty directory {}\n'.format(
                    self.working_space))
        except OSError:
            sys.stdout.write('mkdir {}  directory.\n'.format(
                self.working_space))
            sys.stdout.write('directory is not empty,make sure first:{}.\nexit!\n'.format(
                self.working_space))
            sys.exit()

    def group(self):
        self.groups = defaultdict(list)
        self.working_space = ''
        row_line = 0
        with open(self.input, 'r') as f:
            for i in f:
                if row_line > 0:
                    row_line += 1
                    line = i.strip().split()
                    if len(line) == 5:
                        sample = line[4]
                        self.groups[sample].append(line)
                    else:
                        sys.stdout.write('it seem has less or more than five items in the {} row,it may be a blank row,so pass it and continue\n'.format(
                            row_line))
                else:
                    self.working_space = i.strip().split()
                    assert len(
                        self.working_space) == 1, 'please use batch number,e.g. gmb1,gmb2 ...'
                    self.working_space = self.working_space[0]
                    self.create_working_space()
                    self.working_space = os.path.abspath(self.working_space)
                    row_line += 1

    def info_list_by_sample(self):
        for sample in self.groups:
            info_list = self.groups[sample]
            # each sample may contain more than one library id,these sample will be merge in markduplicate step.
            yield info_list

    def trim(self):
        for info_list in self.info_list_by_sample():
            part = []
            for parallel_sample in info_list:
                fq1, fq2, adapter, library, sample = parallel_sample
                # only accept nextera or truseq adapter
                if adapter.lower() == 'truseq':
                    trim = 'java -jar {tool} PE -threads 14 -phred33 {fq1} {fq2} {library}.{sample}_r1.fq.gz {library}.{sample}_r1_unpaired.fq.gz {library}.{sample}_r2.fq.gz {library}.{sample}_r2_unpaired.fq.gz ILLUMINACLIP:{trimmomatic_dir}/adapters/TruSeq3-PE.fa:2:30:10 LEADING:3 TRAILING:3 SLIDINGWINDOW:4:15 MINLEN:36 &'.format(
                        tool=self.trimmomatic, trimmomatic_dir=os.path.dirname(self.trimmomatic), fq1=fq1, fq2=fq2, library=library, sample=sample)
                elif adapter.lower() == 'nextera':
                    trim = 'java -jar {tool} PE -threads 14 -phred33 {fq1} {fq2} {library}.{sample}_r1.fq.gz {library}.{sample}_r1_unpaired.fq.gz {library}.{sample}_r2.fq.gz {library}.{sample}_r2_unpaired.fq.gz ILLUMINACLIP:{trimmomatic_dir}/adapters/NexteraPE-PE.fa:2:30:10 LEADING:3 TRAILING:3 SLIDINGWINDOW:4:15 MINLEN:36 &'.format(
                        tool=self.trimmomatic, trimmomatic_dir=os.path.dirname(self.trimmomatic), fq1=fq1, fq2=fq2, library=library, sample=sample)
                else:
                    sys.stdout.write('{} :\nadaptor error, only Nextera or Truseq accept,lower or upper dose not matter\n'.format(
                        adapter))
                    sys.exit()
                part.append(trim)
            part.append('wait ;')
            yield part, info_list

    def bwa(self):
        for part, info_list in self.trim():
            for parallel_sample in info_list:
                *_, library, sample = parallel_sample
                bwa = 'mergeps.o {library}.{sample}_r1.fq.gz {library}.{sample}_r2.fq.gz |bwa mem -p -t 14 -R"@RG\\tID:{batch_library}\\tLB:{library}\\tSM:{sample}\\tPU:flowcell\\tPL:Illumina" {reference} - 2>{library}.{sample}.log.bwamem |k8 {bwa_dir}/bwa-postalt.js -p {library}.hla {alt} | samtools sort -@ 2 -m8g - -o {library}.{sample}.aln.bam ;'.format(
                    reference=self.reference, bwa_dir=os.path.dirname(self.seqtk), alt=self.alt, library=library, batch_library=os.path.basename(self.working_space)+'_'+library, sample=sample)
                part.append(bwa)
            yield part, info_list

    def MarkDup(self):
        for part, info_list in self.bwa():
            bams = []
            for parallel_sample in info_list:
                *_, library, sample = parallel_sample
                bams.append("{library}.{sample}.aln.bam".format(
                    library=library, sample=sample))
            MarkDuplicates = '{tool} MarkDuplicates -I {bams}  -O {sample}.dedup.bam -M {sample}.dedup.log -PG null --TMP_DIR ~/tmp/{sample} ;'.format(
                tool=self.gatk, bams=' -I '.join(bams), sample=bams[0].split('.')[1])
            part.append(MarkDuplicates)
            yield part, info_list

    def trim_bwa_dedup(self):
        for part, info_list in self.MarkDup():
            *_, sample = info_list[0]
            index = 'samtools index -@ 4 {sample}.dedup.bam ;'.format(
                sample=sample)
            CollectHsMetrics = '{tool} CollectHsMetrics -I {sample}.dedup.bam -O {sample}_hs_metrics.txt -BI {interval} -TI {interval} -R {reference} &'.format(
                tool=self.gatk, sample=sample, interval=self.interval, reference=self.reference)
            os.mkdir(os.path.join(self.working_space, sample))
            bamdst = '{tool} -p {bed} -o {sample} {sample}.dedup.bam &'.format(
                tool=self.bamdst, bed=self.bed, sample=sample)
            part.append(index)
            part.append(CollectHsMetrics)
            part.append(bamdst)
            sge = "#$ -N {sample}\n#$ -pe smp 16\n#$ -q all.q\n#$ -cwd\nset -e\ncd {working_space}\nsource ~/.bash_profile\n".format(
                sample=sample, working_space=self.working_space)
            with open(os.path.join(self.working_space, '{sample}.trim_bwa_dedup.bat'.format(sample=sample)), 'w') as f:
                f.write(sge)
                f.write('\n'.join(part)+'\n')
                f.write('wait ;')

    def bqsr(self):
        bams = ['{sample}.dedup.bam'.format(
            sample=sample) for sample in self.groups]
        BaseRecalibrator = '{tool} BaseRecalibrator -I {bams} -O recal.table --known-sites {snp} --known-sites {indel} -L {bed} -ip {ip} -R {reference} ;'.format(
            tool=self.gatk, bams=' -I '.join(bams), bed=self.bed, ip=self.ip, snp=self.snp, indel=self.indel, reference=self.reference)
        ApplyBQSR = '{tool} ApplyBQSR -I {bams} -O combined_bqsr.bam -bqsr recal.table -L {bed} -ip {ip} -R {reference} ;'.format(
            tool=self.gatk, bams=' -I '.join(bams), bed=self.bed, ip=self.ip, reference=self.reference)
        part = [BaseRecalibrator, ApplyBQSR]
        sge = "#$ -N {batch}\n#$ -pe smp 16\n#$ -q all.q\n#$ -cwd\nset -e\ncd {working_space}\nsource ~/.bash_profile\n".format(
            working_space=self.working_space, batch=os.path.basename(self.working_space))
        with open(os.path.join(self.working_space, 'bqsr.bat'), 'w') as f:
            f.write(sge)
            f.write('\n'.join(part)+'\n')
            f.write('wait ;')

    def gvcf(self):
        for sample in self.groups:
            HaplotypeCaller = '{tool} HaplotypeCaller -I combined_bqsr.bam --sample-name {sample} -O {sample}.HC.g.vcf.gz --emit-ref-confidence GVCF --dbsnp {dbsnp} -L {bed} -ip {ip} -R {reference} ;'.format(
                tool=self.gatk, sample=sample, bed=self.bed, ip=self.ip, dbsnp=self.dbsnp, reference=self.reference)
            sge = "#$ -N {batch}\n#$ -pe smp 8\n#$ -q all.q\n#$ -cwd\nset -e\ncd {working_space}\nsource ~/.bash_profile\n".format(
                working_space=self.working_space, batch=os.path.basename(self.working_space))
            with open(os.path.join(self.working_space, '{sample}.gvcf.bat'.format(sample=sample)), 'w') as f:
                f.write(sge)
                f.write(HaplotypeCaller+'\n')
                f.write('wait ;')

    def genotype(self):
        cram = "samtools view -C -T {reference} -@ 4 -o {batch}_combined_bqsr.cram combined_bqsr.bam &".format(
            reference=self.reference, batch=os.path.basename(self.working_space))
        gvcfs = [sample+'.HC.g.vcf.gz' for sample in self.groups]
        CombineGVCFs = '{tool} CombineGVCFs -V {gvcfs} -O cohort.g.vcf.gz -R {reference} ;'.format(
            tool=self.gatk, reference=self.reference, gvcfs=' -V '.join(gvcfs))
        GenotypeGVCFs = '{tool} GenotypeGVCFs -V cohort.g.vcf.gz -O han.vcf.gz -R {reference} --dbsnp {dbsnp} -L {bed} -ip {ip} ;'.format(
            tool=self.gatk, dbsnp=self.dbsnp, bed=self.bed, ip=self.ip, reference=self.reference)
        my_snp_Filter = '{tool} SelectVariants -select-type SNP -V han.vcf.gz -O han.snp.vcf.gz ;'.format(
            tool=self.gatk)
        my_snp_Filter2 = '{tool} VariantFiltration -V han.snp.vcf.gz --filter-expression "QD < 2.0 || MQ < 40.0 || FS > 60.0 || SOR > 3.0 || MQRankSum < -12.5 || ReadPosRankSum < -8.0" --filter-name "my_snp_Filter" -O han.snp.hardfilter.vcf.gz ;'.format(
            tool=self.gatk)
        my_indel_Filter = '{tool} SelectVariants -select-type INDEL -V han.vcf.gz -O han.indel.vcf.gz ;'.format(
            tool=self.gatk)
        my_indel_Filter2 = '{tool} VariantFiltration -V han.indel.vcf.gz --filter-expression "QD < 2.0 || FS > 200.0 || ReadPosRankSum < -20.0" --filter-name "my_indel_Filter" -O han.indel.hardfilter.vcf.gz ;'.format(
            tool=self.gatk)
        MergeVcfs = '{tool} MergeVcfs -R {reference} -I han.snp.hardfilter.vcf.gz -I han.indel.hardfilter.vcf.gz -O {batch}.snp.indel.hardfilter.vcf.gz ;'.format(
            tool=self.gatk, batch=os.path.basename(self.working_space), reference=self.reference)
        annotation = 'perl {tool}/table_annovar.pl {batch}.snp.indel.hardfilter.vcf.gz {tool}/humandb/ -buildver hg38 -out {batch} -otherinfo -remove -protocol refGene,cytoBand,exac03,gnomad_exome,clinvar_20170905,dbnsfp33a, -operation g,r,f,f,f,f --vcfinput -nastring . -polish -thread 30'.format(
            tool=self.annovar, batch=os.path.basename(self.working_space))
        self.InterVar_config()
        convert2avinput = 'perl {tool}/convert2annovar.pl -format vcf4old {batch}.snp.indel.hardfilter.vcf.gz >variant.avinput ;'.format(
            tool=self.annovar, batch=os.path.basename(self.working_space))
        ACMG_Intervar = 'python {tool} -c config.ini ;'.format(
            tool=self.Intervar)
        sge = "#$ -N annotation\n#$ -pe smp 16\n#$ -q all.q\n#$ -cwd\nset -e\ncd {working_space}\nsource ~/.bash_profile\n".format(
            working_space=self.working_space)
        part = [cram, CombineGVCFs, GenotypeGVCFs, my_indel_Filter,
                my_indel_Filter2, my_snp_Filter, my_snp_Filter2, MergeVcfs, annotation, convert2avinput, ACMG_Intervar]
        with open(os.path.join(self.working_space, 'genotype.bat'), 'w') as f:
            f.write(sge)
            f.write('\n'.join(part)+'\n')
            f.write('wait ;')

    def InterVar_config(self):
        config = []
        with open('{config}/config.ini'.format(config=os.path.dirname(self.Intervar)), 'r') as f:
            for i in f:
                config.append(i.strip())
        config[3] = 'inputfile=variant.avinput'
        config[6] = 'inputfile_type=avinput'
        config[8] = 'outfile=variant'
        with open('{working_space}/config.ini'.format(working_space=self.working_space), 'w') as f:
            f.write('\n'.join(config))

    def ssh_check(self):
        p = subprocess.Popen('hostname', shell=True, stdout=subprocess.PIPE)
        hostname = p.communicate()[0].strip().decode()
        return hostname

    def run_qsub_step1_step2(self):
        assert self.ssh_check() == 'guomai1', 'please login to guomai1 firstly'
        job_ID = []
        os.chdir(self.working_space)
        sys.stdout.write('changing working directory to your provide {}\n'.format(
            self.working_space))
        if os.path.exists('gm.log'):
            os.remove('gm.log')
        subprocess.call(
            'date +"%Y-%m-%d %H:%M:%S" >> gm.log', shell=True)
        subprocess.call(
            'echo begin all samples trim_bwa_dedup>> gm.log', shell=True)
        for sample in self.groups:
            cmd = 'qsub {sample}.trim_bwa_dedup.bat ;'.format(
                sample=sample)
            print(cmd)
            p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)
            step1_ID = p.communicate()[0].split()[2]
            job_ID.append(step1_ID)
        while 1:
            qstat_ID = self.run_check_by_time()
            _ = set(job_ID) & set(qstat_ID)
            if len(_) == 0:
                subprocess.call(
                    'date +"%Y-%m-%d %H:%M:%S" >> gm.log', shell=True)
                subprocess.call(
                    'echo end all samples trim_bwa_dedup>> gm.log', shell=True)
                cmd = 'qsub bqsr.bat ;'
                sys.stdout.write(cmd+'\n')
                p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)
                step2_ID = p.communicate()[0].split()[2]
                subprocess.call(
                    'date +"%Y-%m-%d %H:%M:%S" >> gm.log', shell=True)
                subprocess.call('echo begin bqsr>> gm.log', shell=True)
                break
            else:
                time.sleep(60)
        return step2_ID

    def run_pbs_step3(self, step2_ID):
        while 1:
            qstat_ID = self.run_check_by_time()
            if step2_ID not in qstat_ID:
                subprocess.call(
                    'date +"%Y-%m-%d %H:%M:%S" >> gm.log', shell=True)
                subprocess.call('echo end bqsr>> gm.log', shell=True)
                files = os.listdir('./')
                pbs = [i for i in files if i.endswith('.gvcf.bat')]
                pbs_job_ID = []
                for sample in pbs:
                    cmd = 'qsub {sample} ;'.format(sample=sample)
                    p = subprocess.Popen(
                        cmd, shell=True, stdout=subprocess.PIPE)
                    ID = p.communicate()[0].split()[2]
                    pbs_job_ID.append(ID)
                    print(cmd)
                subprocess.call(
                    'date +"%Y-%m-%d %H:%M:%S" >> gm.log', shell=True)
                subprocess.call('echo begin gvcf>> gm.log', shell=True)
                break
            else:
                time.sleep(60)
        return pbs_job_ID

    def run_merge_step4(self, pbs_job_ID):
        while 1:
            qstat_ID = self.run_check_by_time()
            _ = set(pbs_job_ID) & set(qstat_ID)
            if len(_) == 0:
                subprocess.call(
                    'date +"%Y-%m-%d %H:%M:%S" >> gm.log', shell=True)
                subprocess.call(
                    'echo completed gvcf and merge vcf,genotyping now>> gm.log', shell=True)
                subprocess.call('qsub genotype.bat', shell=True)
                break
            else:
                time.sleep(60)

    def run_check_by_time(self):
        cmd = "qstat|sed -n '3,$p'|cut -d ' ' -f3"
        p = subprocess.Popen(cmd, stdout=subprocess.PIPE, shell=True)
        qstat_ID = p.communicate()[0].split(b'\n')
        return qstat_ID


def main(p_dict):
    job = p_dict['input']
    result = Wes(job)
    result.trim_bwa_dedup()
    result.bqsr()
    result.gvcf()
    result.genotype()
    if p_dict['run'] == 'on':
        sys.stdout.write('auto qsub will be running in guomai1 right now.\n')
        step2_ID = result.run_qsub_step1_step2()
        pbs_job_ID = result.run_pbs_step3(step2_ID)
        result.run_merge_step4(pbs_job_ID)
    elif p_dict['run'] == 'off':
        sys.stdout.write('cause you choose run off,all job bat will be in {}.\n'.format(
            result.working_space))
    else:
        raise SystemError('only on or off\n')



#!/usr/bin/env python3
# Script to check in WES whether the gender as determined from the bam
# (by counting mappings to chrY normalized to chrX) matches the one specified in gentli
# wdecoster

import sys
import os
import concurrent.futures as cfutures
import subprocess
import re


class Wes_sample(object):
    def __init__(self, samples):
        self.samples = samples
        self.infer_gender()

    def infer_gender(self):
        self.results = {}
        print('SampleName,yreads_count,xreads_count')
        print(self.samples)
        for sample in self.samples:
            # print(self.samples[sample])
            yreads, xreads = map(float, self.samples[sample])
            self.countratio = yreads / xreads
            if self.countratio <= 0.04:
                self.results[sample] = ','.join(
                    map(str, ['f', self.countratio]))
            else:
                self.results[sample] = ','.join(
                    map(str, ['m', self.countratio]))

        with open('infer_gender.txt','w') as f:
            for sample in self.results:
                f.write(sample+':'+self.results[sample])
                print('SampleName: gender,yreads_count/xreads_count')
                print(sample,self.results[sample],sep=':')

        #return self.results

def multi_fetch(bam, sample, bed):
    cmd = 'samtools view -@ 32 -q 5 -F 4 -R {read_group} {bam} {bed}|wc -l'.format(
        read_group=sample+'.txt', bam=bam, bed=bed)
    p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)
    reads = p.communicate()[0]
    return int(reads)


def get_gender(bam):
    chrYregion = 'chrY:2649520-59034050'
    chrXregion = 'chrX:2699520-154931044'
    samples = {}
    '''Determine the gender of a bam file.
    Based on the reads mapping between but not in the PAR regions
    of the Y chromosome normalized to the counts on chromosome X'''
    assert os.path.exists(bam+'.bai') or os.path.exists(re.sub(r'.bam\b',
                                                               '.bai', bam)), 'build index for bam files firstly'
    cmd = "samtools view -H {}|grep '^@RG'|cut -f2|cut -d ':' -f2".format(bam)
    p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)
    sample_list = p.communicate()[0].split(b'\n')
    for sample in sample_list:
        sample = sample.decode()
        if len(sample) != 0:
            #print(sample)
            cmd = 'echo {sample} >{sample}.txt'.format(sample=sample)
            subprocess.call(cmd, shell=True)
            xreads = multi_fetch(bam, sample, chrXregion)
            yreads = multi_fetch(bam, sample, chrYregion)
            cmd = 'rm {}.txt'.format(sample)
            subprocess.call(cmd, shell=True)
            samples[sample] = [yreads, xreads]

    return Wes_sample(samples)


def main(p_dict):
    working_space = p_dict['working_space']
    bamFiles = p_dict['bam']
    #bams = [os.path.realpath(path) for path in glob.glob(sys.argv[1] + '/*.bam')]
    bams = [file for file in os.listdir(
        working_space) if file.endswith(bamFiles)]
    # with cfutures.ProcessPoolExecutor() as executor:
    for bam in bams:
        get_gender(bam)
        #print(res.gender)



#!/usr/bin/env python3
# coding:utf-8
import json
import sys
import pandas as pd
import os


class Gene(object):
    dirpath=os.path.dirname(sys.argv[0])
    package_dirpath=os.path.dirname(dirpath)
    
    def __init__(self, hpo, csv_list):
        self.hpo = hpo
        self.csv_list = csv_list
        self.samples = self.get_csvs()

    def omim_gene(self):  # omim表型编号和基因对应关系
        omim_gene_json=os.path.join(self.package_dirpath,'data/omim_gene.json')
        assert os.path.exists(omim_gene_json),'{} does not exist,it should locate in parent package/data directory'.format(omim_gene_json)
        with open(omim_gene_json, 'r') as f:
            for i in f:
                omim = json.loads(i)
        return omim

    def hpo_phe(self):  # hpo表型编号和omim表型编号对应关系
        omim_phe_hpo_phe_json=os.path.join(self.package_dirpath,'data/omim.phe_hpo.phe.json')
        assert os.path.exists(omim_phe_hpo_phe_json),'{} does not exist,it should locate in parent package/data directory'.format(omim_phe_hpo_phe_json)
        with open(omim_phe_hpo_phe_json, 'r') as f:
            for i in f:
                omim_hpo = json.loads(i)
        return omim_hpo

    def get_hpo(self):  # 获取hpo编号列表
        hpo_list = []
        with open(self.hpo, 'r') as f:
            for i in f:
                line = i.strip().split(':')[-1]
                hpo_list.append(line)
        return hpo_list

    def get_gene(self):
        hpo_list = self.get_hpo()
        omim = self.omim_gene()
        omim_hpo = self.hpo_phe()
        genes = []
        for hpo in hpo_list:
            omim_number_list = omim_hpo.get(hpo, 'no_hpo_number')  # 根据hpo编号获得omim表型编号列表
            if omim_number_list == 'no_hpo_number':
                gene = hpo+'\t'+'no_hpo_number'
                genes.append(gene)
            else:
                for omim_num in omim_number_list:
                    gene_list = omim.get(omim_num, 'no_omim_number')  # 根据omim表型编号获得gene列表
                    if gene_list == 'no_omim_number':
                        gene = hpo+'\t'+omim_num+'\t'+'no_omim_number'
                        genes.append(gene)
                    else:
                        for g in gene_list:
                            gene = hpo+'\t'+omim_num+'\t'+g
                            genes.append(gene)

        genes=list(set(genes))#去重复
        genes=sorted(genes,key=lambda x:int(x.split()[0]))
        return genes

    def write_gene(self):
        self.genes = self.get_gene()
        with open('gene.txt', 'w') as f:
            for i in self.genes:
                f.write(i+'\n')

    def get_csvs(self):
        samples = []
        with open(self.csv_list, 'r') as f:
            for i in f:
                samples.append(i.strip())
        return samples

    def csv_filter(self):
        self.genes = [i.split()[-1] for i in self.genes]
        print (self.genes)
        for sample in self.samples:
            print ('panel.'+sample)
            df = pd.read_csv(sample, header=0,low_memory=False)
            df = df[df['Gene.refGene'].isin(self.genes)]#根据基因列表进行筛选
            out = os.path.basename(sample)
            df.to_csv('panel.'+out, index=False, header=True)


def main(p_dict):
    hpo = p_dict['hpo']
    csv_list = p_dict['csv']
    result = Gene(hpo, csv_list)
    result.write_gene()
    result.csv_filter()



#!/usr/bin/env python3
# coding:utf-8

import gzip
import os
from collections import defaultdict
from collections import OrderedDict
from annotation_filter import Filter
from hgvs import AA_change
from vcf_split import VCFSplit
import pandas as pd
import json
from acmg_rank import Rank
import pysam


class Pedigree(VCFSplit):

    def __init__(self, vcf, rank_dict, child, father, mother):
        super(Pedigree, self).__init__(vcf, rank_dict)
        self.child = child
        self.father = father
        self.mother = mother



    def get_father_mother_GT(self, rec:'VariantFileRecord') -> str:
        for sampleRecorde in rec.samples.values():
            if sampleRecorde.name == self.father:
                father_gt = '/'.join([str(s) for s in sampleRecorde['GT']])
            elif sampleRecorde.name == self.mother:
                mother_gt = '/'.join([str(s) for s in sampleRecorde['GT']])
            else:
                continue
        return father_gt+'|'+mother_gt


    def father_mother_gt_pattern(self) -> dict:
        # get father mother genotype and child genetic pattern
        vcf=pysam.VariantFile(self.vcf)
        sample_list=list(vcf.header.samples)
        assert len(sample_list) == 3,'only support trio sample,the number of sample should equal to 3'
        assert self.child in sample_list and self.father in sample_list and self.mother in sample_list,'child,father,mother mismatch'
        chr_pos_father_mother_GT = defaultdict(dict)
        genetic_pattern_dict = defaultdict(dict)
        for rec in vcf.fetch():
            chr= rec.chrom
            pos=str(rec.pos)
            try:
                genetic_pattern=rec.info['genetic_pattern'][0]
                #print(genetic_pattern)
            except KeyError:
                print('{} should be run with trio module firstly'.format(self.vcf))
            father_mother_GT = self.get_father_mother_GT(rec)
            chr_pos_father_mother_GT[chr][pos] = father_mother_GT
            genetic_pattern_dict[chr][pos] = genetic_pattern

        vcf.close()
        return chr_pos_father_mother_GT, genetic_pattern_dict
            

    def merge(self):
        chr_pos_father_mother_GT, genetic_pattern_dict =self.father_mother_gt_pattern()
        f = ['filter', 'withoutfilter']
        module_child_dir=os.path.join(self.module,self.child)
        print('mergy')
        for filter_type in f:
            child = pd.read_table(os.path.join(module_child_dir,'{module}.{child}.{variant_type}.annovar.{filter_type}.csv'.format(
                module=self.module,child=self.child, variant_type=self.variant_type, filter_type=filter_type)), sep=',', header=0, low_memory=False)

            # 孩子样本需要增加父母的基因型以及遗传模式（genetic pattern）
            col_name = child.columns.tolist()
            col_name.insert(col_name.index('GT')+1, 'father_mother_GT(f|m)')
            col_name.insert(col_name.index('phenotype')+1, 'genetic_pattern')
            child = child.reindex(columns=col_name)
            child['father_mother_GT(f|m)'] = child.apply(lambda x: chr_pos_father_mother_GT[x['chr']][str(x['hg38_position'])], axis=1)
            child['genetic_pattern'] = child.apply(lambda x: genetic_pattern_dict[x['chr']][str(x['hg38_position'])], axis=1)
            # 父母样本不需要genetic_pattern列,父母孩子在先前已经基因型过滤
            child.to_csv(os.path.join(module_child_dir,'{module}.{child}.{variant_type}.annovar.{filter_type}.csv'.format(
                module=self.module,child=self.child, variant_type=self.variant_type, filter_type=filter_type)), index=False, header=True)
            


def main(p_dict):
    vcf = p_dict['vcf']
    intervar = p_dict['rank']
    if intervar == None:
        rank_dict = defaultdict(dict)
    else:
        rank = Rank(intervar)  # 实例化Intervar分级结果
        rank_dict = rank.variant_rank_auto()
    child = p_dict['child']
    father = p_dict['father']
    mother = p_dict['mother']
    variant = ['snp', 'indel']
    result = Pedigree(vcf, rank_dict, child, father, mother)
    for variant_type in variant:
        result.variant_type = variant_type
        result.sample_split()
        result.sample_split(block=False)
        result.merge()



#!/usr/bin/env python3
#coding:utf-8

from collections import defaultdict
import re


class Rank(object):
    def __init__(self,intervar):
        self.intervar=intervar


    def variant_rank_auto(self):
        rank_dict=defaultdict(dict)
        with open(self.intervar,'rt') as f:
            next(f)
            for i in f:
                line=i.split('\t')
                chr=line[0]
                start=line[1]
                rank=line[13].replace(',','\t')
                rank=re.sub(r'^\s+InterVar:\s+','',rank)
                rank_dict['chr'+chr][start]=rank
                #print(rank)
        #print(rank_dict['chr1']['1043223'])
        return rank_dict



#!/usr/bin/env python3
# coding:utf-8


class AA_change(object):
    def __init__(self, value, variant_type):
        self.value = value
        self.variant_type = variant_type


    def std_AA(self):  # 修改AAChange.refGene栏中碱基替换的格式，如果是插入缺失则不改变，仅输出转录本号数值最小的那个
        if self.value == '.' or self.value == 'UNKNOWN':
            match_transcript = '.'
        else:
            value = self.value.split('/')
            if len(value) == 1:
                match_transcript = value[0].split(':')
                match_transcript = self.std(match_transcript)
            else:
                match_transcript = self.get_NM_min(value).split(':')
                match_transcript = self.std(match_transcript)

        return match_transcript

    def std(self, match_transcript):  # 输出标准格式，NM_013296(GPSM2):c.379C>T (p.R127*)
        if self.variant_type == 'snp':
            gene = match_transcript[0]
            transcipt_number = match_transcript[1]
            cDNA = match_transcript[3]
            protein = match_transcript[4]
            _ = cDNA.split('.')[1]
            ori = _[0]
            bed = _[1:-1]
            change = _[-1]
            cDNA = 'c.' + bed + ori + '>' + change
            match_transcript = transcipt_number + \
                '('+gene+')' + ':'+cDNA+'('+protein+')'
        else:
            gene = match_transcript[0]
            transcipt_number = match_transcript[1]
            if len(match_transcript) >= 5:
                cDNA = match_transcript[3]
                protein = match_transcript[4]
                match_transcript = transcipt_number + \
                    '('+gene+')' + ':'+cDNA+'('+protein+')'
            else:
                _ = ':'.join(match_transcript[2:])
                match_transcript = transcipt_number+'('+gene+')' + ':' + _
        return match_transcript

    def get_NM_min(self, value):  # 获得转录本号数值最小的那个转录本
        _ = []
        for nm in value:
            transcipts = nm.split(':')
            number = transcipts[1].split('_')[1]
            pos = [i for i, v in enumerate(number) if v != '0'][0]
            _.append(float(number[pos:]))
        nm_pos = _.index(min(_))
        match_transcript = value[nm_pos]
        return match_transcript

    



#!/usr/bin/env python3
#coding:utf-8

import re


class Filter(object):
    def __init__(self,info_dict,rsid):
        self.info_dict=info_dict
        self.rsid=rsid #self.rsid用于排查错误原因

    def filter_arg(self):
        clinvar = self.info_dict['CLINSIG'].split('|')
        clinvar = list(set(clinvar))
        freq1 = self.info_dict['gnomAD_exome_EAS']
        freq2 = self.info_dict['ExAC_EAS']
        Func_refGene = self.info_dict['Func.refGene']
        # true代表过滤，false代表保留
        if self.clinvar_filter(clinvar):# true意味着仅有benign
            return True  # 仅有benign，过滤，否则下一个判断
        elif self.Pathogenic(clinvar):  # true代表是致病的
            return False  # 致病的不能过滤，否则进行下一个判断
        elif self.population_freq(freq1, freq2):  # true意味着大于0.01
            return True  # 大于0.01就过滤，否则下一个判断
        elif self.Func(Func_refGene):  # true意味着是属于UTR3,intronic
            return True  # 属于UTR3等过滤，否则保留
        else:
            return False  # benign不是唯一值,又不是致病的，但是频率小于0.01，又不属于UTR3等，这些位点保留

    def clinvar_filter(self, clinvar):  # 删除仅有benign或者likely_benign
            """other|Benign"""
            """Pathogenic或Likely Pathogenic"""
            length = len(clinvar)
            if length == 1 and (clinvar[0] == 'Benign' or clinvar[0] == 'Likely_benign'):
                return True
            elif length == 2 and ((clinvar[0] == 'Benign' or clinvar[1] == 'Likely_benign') or (clinvar[0] == 'Likely_benign' or clinvar[1] == 'Benign')):
                return True
            else:
                False

    def Pathogenic(self, clinvar):
        clinvar_re=re.findall(r'Pathogenic','|'.join(clinvar))
        if len(clinvar_re) >= 1: # >=1意味着是致病的，['Pathogenic']
            return True
        else:
            return False


    def population_freq(self, freq1, freq2):
        if freq1 == '.':
            freq1 = 0
        if freq2 == '.':
            freq2 = 0
        freq = max(float(freq1), float(freq2))
        if freq < 0.01:
            return False
        else:
            return True

    def Func(self, Func_refGene):
        if Func_refGene == 'UTR3' or Func_refGene == 'UTR5' or Func_refGene == 'ncRNA':
            return True
        else:
            return False

    

    



#!/usr/bin/env python2
# coding:utf-8

import subprocess
from collections import defaultdict
import gzip
import os,sys


class Hg38ToHg19(object):
    def __init__(self, vcf):
        self.vcf = vcf
        self.write_bed()
        self.liftOver()


    def write_bed(self):
        if self.vcf.endswith('.gz'):
            input = gzip.open(self.vcf, 'rt')
        else:
            input = open(self.vcf, 'rt')
        with open('hg38.bed', 'wt') as f:
            for i in input:
                if i.startswith('#'):
                    continue
                else:
                    line = i.split()
                    chr = line[0]
                    pos = line[1]
                    out = [chr, int(pos)-1, pos, chr, pos]
                    out = map(str, out)
                    f.write('\t'.join(out)+'\n')
        input.close()
        # cmd = "less  {vcf} |grep -v '^#'|awk '{arg}' >hg38.bed".format(
        #     vcf=self.vcf, arg='{print $1,$2-1,$2,$1,$2}')
        # subprocess.call(cmd, shell=True)
        return None

    def liftOver(self):
        dirpath=os.path.dirname(sys.argv[0])
        package_dirpath=os.path.dirname(dirpath)
        hg38ToHg19_over_chain=os.path.join(package_dirpath,'data/hg38ToHg19.over.chain.gz')
        assert os.path.exists(hg38ToHg19_over_chain),'{} does not exist,it should locate in parent package/data directory'.format(hg38ToHg19_over_chain)
        cmd = 'liftOver hg38.bed {} hg19.bed unmap'.format(hg38ToHg19_over_chain)
        subprocess.call(cmd, shell=True)
        return None

    def get_bed(self):
        with open('hg19.bed', 'rt') as f:
            for i in f:
                line = i.strip().split()
                yield line

    def hg19_position(self):
        hg38_hg19 = defaultdict(dict)
        for line in self.get_bed():
            chr = line[3]
            hg38_position = line[4]
            _hg19_position = line[2]
            hg38_hg19[chr][hg38_position] = _hg19_position
        return hg38_hg19



#!/usr/bin/env python3
# coding:utf-8

import pysam
import os
import re
import sys
from collections import defaultdict
from collections import OrderedDict
from hg38Tohg19 import Hg38ToHg19
from hgvs import AA_change
from annotation_filter import Filter
import json
from acmg_rank import Rank
#import pysnooper
#from stat_check import Stat
import logging


class VCFSplit(object):
    title1 = ['chr', 'hg38_position', 'hg19_position', 'rsid', 'ref', 'alt', 'qual', 'filter',
              'GT', 'RefDepth', 'Alt1Depth', 'Alt2Depth', 'totalDP', 'genotype_quality', 'match_transcript', 'inheritance', 'phenotype', 'ACMG_rank']

    def __init__(self, vcf:'VariantFile', rank_dict:dict,variant_type = 'snp'):
        self.vcf = vcf
        self.rank_dict = rank_dict
        self.variant_type = variant_type
        self.hg38_hg19 = self.coordinate()
        self.title2 = self.get_title2()
        self.title = self.title1 + self.title2
        print(self.title,sep='\t')
        self.gene_phe = self.gene_phenotype()


    def coordinate(self) -> dict:
        if not hasattr(self, "hg38_hg19"):
            hg38_hg19 = Hg38ToHg19(self.vcf).hg19_position()
        return hg38_hg19

    def get_title2(self) -> list :
        # title2=[]# 对应下面输出文件表头的第二部分
        # title2(Func.refGene->MutationAssessor_pred)
        vcf = pysam.VariantFile(self.vcf)
        # self.vcf_header=re.findall(r'(#CHROM.*)',str(vcf.header))[0]
        if not hasattr(self, 'title2'):
            for rec in vcf.fetch():
                info = rec.info.keys()
                start, end = info.index('Func.refGene'), info.index(
                    'MutationAssessor_pred')
                title2 = info[start:end+1]
                break
        vcf.close()
        return title2

    def snp_or_indel(self, ref:str, alt:tuple) -> str:
        # infer variant type
        #alt = alt.split('/')
        alt = sorted(alt, key=lambda x: len(x), reverse=False)[-1]
        if len(ref) > 1 or len(alt) > 1:
            return 'indel'
        else:
            return 'snp'

    def indel_pos_allele(self, pos:int, ref:str, alt:tuple) -> str:
        # there could be more than two alternative allele
        # get indel true position,return type is string
        alt = re.split(r'([^\/]+)\/([\w\/]+)', '/'.join(alt))
        if '' in alt:
            alt.remove('')
        alt = alt[0]
        if len(ref) > len(alt):
            head = ref[0:len(alt)]
            if head == alt:
                start = str(pos+len(head))
            else:
                start = pos
        else:
            head = alt[0:len(ref)]
            if head == ref:
                start = str(pos+len(ref)-1)
            else:
                start = pos
        return str(start)

    #@pysnooper.snoop('test.log')
    def parse_vcf(self, cut=True) -> None:
        # get sample_dict,contain samples record
        #self.rsid用于排查错误原因
        #tt=time.clock()
        self.sample_dict = defaultdict(list)
        vcf = pysam.VariantFile(self.vcf)
        for rec in vcf.fetch():
            chr, pos, self.rsid, ref, alt, qual = rec.chrom, rec.pos, rec.id, rec.ref, rec.alts, rec.qual
            hg19_position = self.hg38_hg19[chr].get(str(pos), 'delete')
            if self.variant_type == self.snp_or_indel(ref, alt):
                if self.variant_type == 'indel':
                    start = self.indel_pos_allele(pos, ref, alt)
                else:
                    start = str(pos)
                ACMG_rank = self.rank_dict.get(chr,{}).get(start, '.')
                filter_ = rec.filter.keys()[0]
                if filter_ == 'PASS':
                    basic_data = [chr, pos, hg19_position,
                                  self.rsid, ref, '/'.join(alt), qual, filter_]
                    info = rec.info.keys()
                    # 寻找Func.refGene作为开始点，MutationAssessor_pred作为终止点
                    start, end = info.index('Func.refGene'), info.index('MutationAssessor_pred')
                    info = info[start:end+1]
                    info_dict = OrderedDict()
                    for key in info:
                        value = '/'.join(rec.info[key]).replace(',', '/')
                        info_dict[key] = str(value)
                    info_dict_OrderedValues = list(info_dict.values())
                    AA_value = info_dict['AAChange.refGene']
                    match_transcript = AA_change(AA_value, self.variant_type).std_AA()
                    gene = info_dict['Gene.refGene']
                    inheritance, phenotype = self._gene_phe(gene)
                    omim = [match_transcript, inheritance, phenotype, ACMG_rank]
                    if cut:  # true代表对文件过滤，false表示不过滤
                        # 为真就过滤掉不符合条件的变异结果，反之结果保留，判断语句在annotation_filter.py中
                        __filter = Filter(info_dict,self.rsid)
                        if __filter.filter_arg():
                            # if self.rsid == 'rs1799983':
                            #     print(1)
                            continue
                        else:
                            self.sample_record_filter(
                                rec, basic_data, omim, info_dict_OrderedValues)
                    else:
                        self.sample_record_filter(
                            rec, basic_data, omim, info_dict_OrderedValues)

        vcf.close()
        print(self.variant_type,'filter={}'.format(cut))
        return None
        #ee=time.clock()
        #print(ee-tt)

    def sample_record_filter(self, rec:'VariantFileRecord', basic_data:list, omim:list, info_dict_OrderedValues:list) :
        #  update self.sample_dict
        for sampleRecord in rec.samples.values():
            try:
                GT = sampleRecord['GT']
                AD = sampleRecord['AD']
                genotype_quality = sampleRecord['GQ'] #int
                totalDP = sampleRecord['DP']
                if GT.__eq__((0, 0)) or GT.__eq__((None, None)) or GT.__eq__((None,)):
                    continue
                else:
                    try:
                        RefDepth = AD[0]
                        Alt1Depth = AD[1]
                    except IndexError:
                        continue
                    if totalDP == None or genotype_quality == None or genotype_quality < 20:
                        continue
                    else:
                        Alt2Depth = int(totalDP)-int(RefDepth)-int(Alt1Depth)
                    sample_data = ['/'.join(str(s) for s in GT), RefDepth, Alt1Depth,
                                   Alt2Depth, totalDP, genotype_quality]
                    all = basic_data+sample_data+omim+info_dict_OrderedValues
                    self.sample_dict[sampleRecord.name].append(all)

            except KeyError:
                continue
                
    def gene_phenotype(self) -> dict:  # 匹配基因对应的表型和遗传方式，AD,AR,XL
        # gene:[inheritance,phenotype]
        dirpath = os.path.dirname(sys.argv[0])
        package_dirpath = os.path.dirname(dirpath)
        inh_gene_phenotype_json = os.path.join(
            package_dirpath, 'data/inh_gene_phenotype.json')
        assert os.path.exists(
            inh_gene_phenotype_json), '{} does not exist,it should locate in parent package/data directory'.format(inh_gene_phenotype_json)
        if not hasattr(self, 'gene_phe'):
            with open(inh_gene_phenotype_json, 'r') as f:
                for i in f:
                    gene_phe = json.loads(i)
        return gene_phe

    def _gene_phe(self, gene) -> tuple:
        inh_phe = self.gene_phe.get(gene, '.')
        if inh_phe == '.':
            inheritance, phenotype = '.', '.'
        else:
            inheritance, phenotype = inh_phe
        inheritance = inheritance.replace(',', '/')
        phenotype = phenotype.replace(',', '_')
        return inheritance, phenotype

    def output_name(self, cut=True) -> str:
        if cut:
            output = '.{}.annovar.filter.csv'.format(self.variant_type)
        else:
            output = '.{}.annovar.withoutfilter.csv'.format(self.variant_type)
        return output

    #@pysnooper.snoop('test.log')
    def sample_split(self, cut=True):
        self.module = sys.argv[1]
        self.parse_vcf(cut)
        now_path=os.path.abspath('./')
        try:
            print('you can find the csv result in {} directory'.format(self.module))
            os.mkdir('{}'.format(self.module))
        except OSError:
            print('carrier exists,reuse it')
            pass
        os.chdir('{}'.format(self.module))
        for sample in self.sample_dict:
            try:
                os.mkdir(sample)
            except OSError:
                print('{} exists,reuse it'.format(sample))
                pass
            output = self.module + '.' + sample + self.output_name(cut)
            print('create {}'.format(output))
            with open(os.path.join(sample,output), 'wt') as f:
                f.write(','.join(self.title)+'\n')
                for sample_list in self.sample_dict[sample]:
                    sample_list = map(str, sample_list)
                    f.write(','.join(sample_list)+'\n')
        os.chdir(now_path)


def main(p_dict:dict):
    logging.basicConfig(level=logging.CRITICAL,filename='carrier.log')
    logging.critical('')
    #Stat().stat()
    vcf = p_dict['vcf']
    intervar = p_dict['rank']
    if intervar == None:
        rank_dict = defaultdict(dict)
    else:
        rank = Rank(intervar)  # 实例化Intervar分级结果
        rank_dict = rank.variant_rank_auto()
    variant = ['snp', 'indel']
    result = VCFSplit(vcf, rank_dict)
    for variant_type in variant:
        result.variant_type = variant_type
        result.sample_split()
        result.sample_split(cut=False)
    del result.sample_dict
    #Stat().stat()
    





from setuptools import setup
from setuptools import find_packages
from os import path
import os

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'file/README.md'),'r') as f:
    long_description = f.read()
files=os.listdir('bin/')
files=['bin/'+i for i in files]
files.remove('bin/__init__.py')
setup(name='pywes',version='0.0.2',description='carrier and trio analysis',long_description=long_description, url='https://github.com/N-damo/exomreport',
author="Li'an Lin",author_email='21620151153308@stu.xmu.edu.cn',license='GPL3', keywords='exom', 
include_package_data=True,packages=['bin'],scripts=files,classifiers=['License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)'])



#!/usr/bin/env python3
#coding:utf-8

import sys

def diff(test,correct):
    try:
        test_lines = open(test).readlines()
        correct_lines = open(correct).readlines()
        for test, correct in zip(test_lines, correct_lines):
            if test != correct:
                print ("Oh no! Expected %r; got %r." % (correct, test))
                break
            else:
                len_diff = len(test_lines) - len(correct_lines)
                if len_diff > 0:
                    print ("Test file had too much data.")
                elif len_diff < 0:
                    print ("Test file had too little data.")
                else:
                    print ("Everything was correct!")
    except IOError:
        print('file does not exist')
        sys.exit()

if __name__ == '__main__':
    test,correct=sys.argv[1],sys.argv[2]
    diff(test,correct)